@@ -30,7 +30,7 @@ class KeywordsExtractor:
             if self.word_filter.filter(word):
                 continue
             freq[word] += count
-        for word, count, _ in long_words:
+        for word, count in long_words:
             if self.word_filter.filter(word):
                 continue
             freq[word] += count
@@ -40,7 +40,7 @@ class KeywordsExtractor:
 
         # FIXME: 根据经验设置的权重阈值
         min_weight = 0.1
-        res = [k for k in res if k[1] >= min_weight]
+        res = [[k, v] for k, v in res if v >= min_weight]
         return res
 
     def _tokenize(self, doc: Union[str, List[str]]) -> List[str]:
@@ -56,66 +56,79 @@ class KeywordsExtractor:
         return words
 
     def _filter_subwords(self, words: list, freq: Dict[str, int]):
-        not_merge = words
         merged = []
-        for i in range(len(not_merge)):
-            w1 = not_merge[i][0]
+        for i in range(len(words)):
+            w1 = words[i][0]
             w1_ok = True
-            for j in range(len(not_merge)):
-                w2 = not_merge[j][0]
+            for j in range(len(words)):
+                w2 = words[j][0]
                 if len(w1) < len(w2) and w1 in w2 and freq[w1] == freq[w2]:
                     w1_ok = False
                     break
             if w1_ok:
-                merged.append(not_merge[i])
+                merged.append((words[i][0], words[i][1]))
         return merged
 
-    def _detect_long_keywords(self, words, keywords):
-        keywords = set([t[0] for t in keywords])
-        appears = defaultdict(lambda: 0)
-        parts = defaultdict(list)
+    def _detect_long_keywords(self, words, short_words):
+        keywords = set([k for k, _ in short_words])
+        appears = {}
 
-        def commit(continuous_words):
-            end_pos = len(continuous_words)
-            if end_pos > 0:
-                for i in range(0, end_pos - 1):
-                    for j in range(i + 2, end_pos + 1):
-                        word = ''.join(continuous_words[i:j])
-                        appears[word] += 1
-                        parts[word] = continuous_words[i:j]
-
-        w = []
-        for cur in range(0, len(words)):
-            x = words[cur]
-            if x in keywords:
-                w.append(x)
-            elif w:
-                commit(w)
-                w = []
-        if w:
-            commit(w)
+        a = []
+        for w in words:
+            if w in keywords:
+                a.append(w)
+            elif a:
+                self._commit_continuous_words(a, appears)
+                a = []
+        if a:
+            self._commit_continuous_words(a, appears)
 
         long_words = []
         for word, count in appears.items():
             # FIXME: 合理计算阈值
             if count >= 2:
-                long_words.append((word, count, parts[word]))
+                long_words.append((word, count))
         return long_words
 
+    def _commit_continuous_words(self, continuous_words: list, appears: dict):
+        end_pos = len(continuous_words)
+        if end_pos > 0:
+            for i in range(0, end_pos - 1):
+                for j in range(i + 2, end_pos + 1):
+                    word = ''.join(continuous_words[i:j])
+                    if word in appears:
+                        appears[word] += 1
+                    else:
+                        appears[word] = 1
+
     def _extract_short_keywords(self, words) -> list:
-        freq = defaultdict(lambda: 0)
+        freq = {}
         for w in words:
             if self.stopwords.is_stopword(w):
                 continue
-            freq[w] += 1
+            if w in freq:
+                freq[w] += 1
+            else:
+                freq[w] = 1
+
         tfidf = {}
         for k in freq:
             tfidf[k] = freq[k] * self.idf.word_idf(k)
-        keywords = sorted(tfidf, key=tfidf.__getitem__, reverse=True)
-        keywords = [(k, freq[k]) for k in keywords if freq[k] > 1]
+
+        t = []
+        for w in tfidf:
+            t.append((w, tfidf[w]))
+        t.sort(key=lambda x: x[1], reverse=True)
+
         topk = 500
         # FIXME: 根据文本长度等信息动态调整简单关键词的数量
-        return keywords[:topk]
+        result = []
+        for w, _ in t:
+            if freq[w] > 1:
+                result.append((w, freq[w]))
+            if len(result) >= topk:
+                break
+        return result
 
     def _rerank_words(self, freq: Dict[str, int]) -> list:
         res = []
